///////////////////////////////////////////////////////////////////////
//                                                                   //
//   MainForm.cs                                                     //
//                                                                   //
//   C# source code for main application window.                     //
//                                                                   //
//   Copyright (C) 2007, by Matt Roberts, KK5JY,                     //
//   All rights reserved.                                            //
//                                                                   //
///////////////////////////////////////////////////////////////////////

// system modules
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace SimpleImageViewer {
	//
	//
	//   class MainForm - form upon which the rest of the application is built
	//
	//
	public partial class MainForm : Form {
		#region Internal Fields

		// list of image files
		private string[] loopImages;

		// interval timer for looping
		private Timer loopTimer;

		// current image index into loopImages
		private int loopIndex;

		#endregion

		#region Public Instance Constructors

		// default ctor
		public MainForm() {
			// initialize designer half of form
			InitializeComponent();

			// initialize loop stuff
			loopImages = null;

			// aspect handling
			imagePanel.PreserveAspectRatio = m_PreserveAspect.Checked;

			// initialize the timer
			loopTimer = new Timer();
			loopTimer.Interval = 500;
			loopTimer.Tick += this.OnTick;

			// adjust menus
			AdjustMenus();
		}

		#endregion

		#region Public Properties

		// test whether one or more files are open for viewing/looping
		public bool IsFileOpen {
			get { return loopImages != null; }
		}

		// test whether the window is current looping
		public bool Looping {
			get { return loopTimer != null && loopImages.Length > 0; }
		}

		#endregion

		#region Public Methods

		//
		//
		//   SetImage
		//
		//
		public void SetImage(string file) {
			if (file == null) {
				imagePanel.Image = null;
				return;
			}

			Cursor c = Cursor.Current;
			Cursor.Current = Cursors.WaitCursor;
			try {
				Image oldImage = imagePanel.Image;
				imagePanel.Image = new Bitmap(file);
				if (oldImage != null)
					oldImage.Dispose();
			} finally {
				Cursor.Current = c;
			}
		}


		//
		//
		//   UpdateImage()
		//
		//
		public void UpdateImage() {
			if (loopImages == null) {
				SetImage(null);
			}

			if (loopIndex >= loopImages.Length) {
				SetImage(null);
			} else {
				SetImage(loopImages[loopIndex]);
			}

			SetStatus();
		}

		
		//
		//
		//   AdjustMenus
		//
		//
		public void AdjustMenus() {
			m_Loop.Enabled = IsFileOpen && !loopTimer.Enabled;
			b_Loop.Enabled = IsFileOpen && !loopTimer.Enabled;
			m_Stop.Enabled = IsFileOpen && loopTimer.Enabled;
			b_Stop.Enabled = IsFileOpen && loopTimer.Enabled;
		}


		//
		//
		//   StartLoop
		//
		//
		private void StartLoop() {
			if (loopImages.Length <= 1) {
				return;
			}

			// enable the timer
			loopTimer.Start();

			// adjust menus
			AdjustMenus();
		}


		//
		//
		//   SetStatus
		//
		//
		public void SetStatus() {
			if (loopImages != null && loopImages.Length > 0) {
				if (loopIndex >= 0) {
					statusLabel.Text = String.Format("Showing image {0} of {1}.", loopIndex + 1, loopImages.Length);
					progressBar.Value = (100 * loopIndex / loopImages.Length);
				} else {
					statusLabel.Text = String.Format("{0} images selected.", loopImages.Length);
					progressBar.Value = 0;
				}
			} else {
				statusLabel.Text = "No images loaded";
				progressBar.Value = 0;
			}
		}

		#endregion

		#region Event Handlers

		//
		//
		//   Help-About
		//
		//
		private void ShowAbout(object sender, EventArgs e) {
			AboutBox dialog = new AboutBox();
			try {
				dialog.ShowDialog();
			} finally {
				dialog.Dispose();
			}
		}


		//
		//
		//   File-Open
		//
		//
		private void FileMenuOpenFiles(object sender, EventArgs e) {
			OpenFileDialog dialog = new OpenFileDialog();
			dialog.Multiselect = true;
			dialog.CheckFileExists = false;
			dialog.CheckPathExists = false;
			try {
				DialogResult result = dialog.ShowDialog();
				if (result != DialogResult.OK)
					return;
				if (dialog.FileNames.Length == 0)
					return;

				// stop the loop
				loopTimer.Stop();

				// store the list of files
				loopImages = dialog.FileNames;

				// load the first file
				loopIndex = 0;
				SetImage(loopImages[loopIndex]);

				// update status bar
				SetStatus();

			} finally {
				dialog.Dispose();
			}

			AdjustMenus();
		}


		//
		//
		//   File-Close
		//
		//
		private void FileMenuClose(object sender, EventArgs e) {
			if (IsFileOpen) {
				loopTimer.Stop();
				loopImages = null;
				SetImage(null);
				SetStatus();
			} else {
				FileMenuExit(sender, e);
			}

			AdjustMenus();
		}

		
		//
		//
		//   File-Exit
		//
		//
		private void FileMenuExit(object sender, EventArgs e) {
			Close();
		}


		//
		//
		//   View-Loop
		//
		//
		private void ViewMenuLoop(object sender, EventArgs e) {
			StartLoop();
		}


		//
		//
		//   Options-Loop_Interval
		//
		//
		private void OptionsMenuSetLoopInterval(object sender, EventArgs e) {
			LoopIntervalDialog dialog = new LoopIntervalDialog();
			dialog.Interval = loopTimer.Interval;
			try {
				DialogResult result = dialog.ShowDialog();
				if (result != DialogResult.OK)
					return;

				if (dialog.Interval >= 50) {
					loopTimer.Interval = dialog.Interval;
				}

			} finally {
				dialog.Dispose();
			}
		}


		//
		//
		//   Options-Preserve_Aspect_Ratio
		//
		//
		private void OptionsMenuPreserveAspect(object sender, EventArgs e) {
			m_PreserveAspect.Checked =
				!m_PreserveAspect.Checked;
			imagePanel.PreserveAspectRatio =
				m_PreserveAspect.Checked;
		}


		//
		//
		//   OnClosing
		//
		//
		protected override void OnClosing(CancelEventArgs e) {
			if (loopTimer != null) {
				loopTimer.Stop();
			}

			// chain to base
			base.OnClosing(e);
		}


		//
		//
		//   OnTick
		//
		//
		private void OnTick(object sender, EventArgs e) {
			// disable the timer
			loopTimer.Enabled = false;

			if (++loopIndex >= loopImages.Length)
				loopIndex = 0;

			Bitmap newImage = null;
			bool doContinue = true;
			do {
				try {
					newImage = new Bitmap(loopImages[loopIndex]);
					doContinue = false;
					UpdateImage();
				} catch (Exception) {
					doContinue = true;
					if (newImage != null) {
						newImage.Dispose();
					}
				}
			} while (doContinue);

			// update the status bar
			SetStatus();

			// reenable the timer
			loopTimer.Enabled = true;
		}


		//
		//
		//   OnKeyDown
		//
		//
		private void KeyDownHandler(object sender, KeyEventArgs e) {
			if (IsFileOpen) {
				if (e.Modifiers == Keys.None) {
					// catch loop control keys
					switch (e.KeyCode) {
						//
						//  ESC
						//
						case Keys.Escape:
							e.SuppressKeyPress = true;
							loopTimer.Stop();
							break;
						//
						//  SPACE
						//
						case Keys.Space:
							e.SuppressKeyPress = true;
							loopTimer.Enabled = !loopTimer.Enabled;
							break;
						//
						//  LEFT Arrow
						//
						case Keys.Left:
							e.SuppressKeyPress = true;
							if (loopTimer.Enabled) {
								loopTimer.Stop();
							} else {
								--loopIndex;
								if (loopIndex < 0)
									loopIndex = loopImages.Length - 1;
								UpdateImage();
							}
							break;
						//
						//  LEFT Arrow
						//
						case Keys.Right:
							e.SuppressKeyPress = true;
							if (loopTimer.Enabled) {
								loopTimer.Stop();
							} else {
								++loopIndex;
								if (loopIndex >= loopImages.Length)
									loopIndex = 0;
								UpdateImage();
							}
							break;
						//
						//  UP Arrow
						//
						case Keys.Up:
							e.SuppressKeyPress = true;
							if (loopTimer.Interval > 50) {
								loopTimer.Interval -= 10;
								if (loopTimer.Interval < 50) {
									loopTimer.Interval = 50;
								}
							}
							break;
						//
						//  DOWN Arrow
						//
						case Keys.Down:
							e.SuppressKeyPress = true;
							if (loopTimer.Interval < 1000) {
								loopTimer.Interval += 10;
								if (loopTimer.Interval > 1000) {
									loopTimer.Interval = 1000;
								}
							}
							break;
					}
				}
			}
		}

		#endregion

		private void b_Open_Click(object sender, EventArgs e) {
			FileMenuOpenFiles(sender, e);
		}

		private void b_LoopFiles_Click(object sender, EventArgs e) {
			ViewMenuLoop(sender, e);
		}

		private void b_Stop_Click(object sender, EventArgs e) {
			loopTimer.Stop();
		}
	}
}